@@ -36,7 +36,9 @@
 
 #include <gbm.h>
 
+#ifndef HAVE_GLAMOR_FINISH
 #include <GL/gl.h>
+#endif
 
 DevPrivateKeyRec amdgpu_pixmap_index;
 
@@ -64,10 +66,9 @@ Bool amdgpu_glamor_create_screen_resources(ScreenPtr screen)
 #endif
 
 	if (!amdgpu_bo_get_handle(info->front_buffer, &bo_handle) ||
-	    !glamor_egl_create_textured_screen_ext(screen,
-						   bo_handle,
-						   scrn->displayWidth *
-						   info->pixel_bytes, NULL)) {
+	    !glamor_egl_create_textured_screen(screen, bo_handle,
+					       scrn->displayWidth *
+					       info->pixel_bytes)) {
 		return FALSE;
 	}
 
@@ -80,9 +81,6 @@ Bool amdgpu_glamor_pre_init(ScrnInfoPtr scrn)
 	pointer glamor_module;
 	CARD32 version;
 
-	if (!info->dri2.available)
-		return FALSE;
-
 	if (scrn->depth < 24) {
 		xf86DrvMsg(scrn->scrnIndex, X_ERROR,
 			   "glamor requires depth >= 24, disabling.\n");
@@ -127,7 +125,7 @@ Bool amdgpu_glamor_pre_init(ScrnInfoPtr scrn)
 }
 
 Bool
-amdgpu_glamor_create_textured_pixmap(PixmapPtr pixmap, struct amdgpu_pixmap *priv)
+amdgpu_glamor_create_textured_pixmap(PixmapPtr pixmap, struct amdgpu_buffer *bo)
 {
 	ScrnInfoPtr scrn = xf86ScreenToScrn(pixmap->drawable.pScreen);
 	AMDGPUInfoPtr info = AMDGPUPTR(scrn);
@@ -136,7 +134,7 @@ amdgpu_glamor_create_textured_pixmap(PixmapPtr pixmap, struct amdgpu_pixmap *pri
 	if ((info->use_glamor) == 0)
 		return TRUE;
 
-	if (!amdgpu_bo_get_handle(priv->bo, &bo_handle))
+	if (!amdgpu_bo_get_handle(bo, &bo_handle))
 		return FALSE;
 
 	return glamor_egl_create_textured_pixmap(pixmap, bo_handle,
@@ -233,7 +231,7 @@ amdgpu_glamor_create_pixmap(ScreenPtr screen, int w, int h, int depth,
 
 		pixmap->devPrivate.ptr = NULL;
 
-		if (!amdgpu_glamor_create_textured_pixmap(pixmap, priv))
+		if (!amdgpu_glamor_create_textured_pixmap(pixmap, priv->bo))
 			goto fallback_glamor;
 	}
 
@@ -303,10 +301,9 @@ amdgpu_glamor_set_pixmap_bo(DrawablePtr drawable, PixmapPtr pixmap)
 		FreeScratchGC(gc);
 	}
 
-	amdgpu_set_pixmap_private(pixmap, NULL);
-
 	/* And redirect the pixmap to the new bo (for 3D). */
 	glamor_egl_exchange_buffers(old, pixmap);
+	amdgpu_set_pixmap_private(pixmap, amdgpu_get_pixmap_private(old));
 	amdgpu_set_pixmap_private(old, priv);
 
 	screen->ModifyPixmapHeader(old,
@@ -320,20 +317,27 @@ amdgpu_glamor_set_pixmap_bo(DrawablePtr drawable, PixmapPtr pixmap)
 	return old;
 }
 
-#ifdef AMDGPU_PIXMAP_SHARING
 
 static Bool
 amdgpu_glamor_share_pixmap_backing(PixmapPtr pixmap, ScreenPtr slave,
 				   void **handle_p)
 {
 	ScreenPtr screen = pixmap->drawable.pScreen;
+	AMDGPUInfoPtr info = AMDGPUPTR(xf86ScreenToScrn(screen));
 	uint64_t tiling_info;
 	CARD16 stride;
 	CARD32 size;
+	Bool is_linear;
 	int fd;
 
 	tiling_info = amdgpu_pixmap_get_tiling_info(pixmap);
-	if (AMDGPU_TILING_GET(tiling_info, ARRAY_MODE) != 0) {
+
+	if (info->family >= AMDGPU_FAMILY_AI)
+		is_linear = AMDGPU_TILING_GET(tiling_info, SWIZZLE_MODE) == 0;
+	else
+		is_linear = AMDGPU_TILING_GET(tiling_info, ARRAY_MODE) == 1;
+
+	if (!is_linear) {
 		PixmapPtr linear;
 
 		/* We don't want to re-allocate the screen pixmap as
@@ -372,7 +376,7 @@ amdgpu_glamor_set_shared_pixmap_backing(PixmapPtr pixmap, void *handle)
 
 	priv = amdgpu_get_pixmap_private(pixmap);
 
-	if (!amdgpu_glamor_create_textured_pixmap(pixmap, priv)) {
+	if (!amdgpu_glamor_create_textured_pixmap(pixmap, priv->bo)) {
 		xf86DrvMsg(scrn->scrnIndex, X_ERROR,
 			   "Failed to get PRIME drawable for glamor pixmap.\n");
 		return FALSE;
@@ -386,7 +390,6 @@ amdgpu_glamor_set_shared_pixmap_backing(PixmapPtr pixmap, void *handle)
 	return TRUE;
 }
 
-#endif /* AMDGPU_PIXMAP_SHARING */
 
 Bool amdgpu_glamor_init(ScreenPtr screen)
 {
@@ -443,13 +446,11 @@ Bool amdgpu_glamor_init(ScreenPtr screen)
 	screen->CreatePixmap = amdgpu_glamor_create_pixmap;
 	info->glamor.SavedDestroyPixmap = screen->DestroyPixmap;
 	screen->DestroyPixmap = amdgpu_glamor_destroy_pixmap;
-#ifdef AMDGPU_PIXMAP_SHARING
 	info->glamor.SavedSharePixmapBacking = screen->SharePixmapBacking;
 	screen->SharePixmapBacking = amdgpu_glamor_share_pixmap_backing;
 	info->glamor.SavedSetSharedPixmapBacking = screen->SetSharedPixmapBacking;
 	screen->SetSharedPixmapBacking =
 	    amdgpu_glamor_set_shared_pixmap_backing;
-#endif
 
 	xf86DrvMsg(scrn->scrnIndex, X_INFO, "Use GLAMOR acceleration.\n");
 	return TRUE;
@@ -461,8 +462,9 @@ void amdgpu_glamor_flush(ScrnInfoPtr pScrn)
 
 	if (info->use_glamor) {
 		glamor_block_handler(pScrn->pScreen);
-		info->gpu_flushed++;
 	}
+
+	info->gpu_flushed++;
 }
 
 void amdgpu_glamor_finish(ScrnInfoPtr pScrn)
@@ -470,8 +472,13 @@ void amdgpu_glamor_finish(ScrnInfoPtr pScrn)
 	AMDGPUInfoPtr info = AMDGPUPTR(pScrn);
 
 	if (info->use_glamor) {
+#if HAVE_GLAMOR_FINISH
+		glamor_finish(pScrn->pScreen);
+		info->gpu_flushed++;
+#else
 		amdgpu_glamor_flush(pScrn);
 		glFinish();
+#endif
 	}
 }
 
@@ -485,10 +492,8 @@ amdgpu_glamor_fini(ScreenPtr screen)
 
 	screen->CreatePixmap = info->glamor.SavedCreatePixmap;
 	screen->DestroyPixmap = info->glamor.SavedDestroyPixmap;
-#ifdef AMDGPU_PIXMAP_SHARING
 	screen->SharePixmapBacking = info->glamor.SavedSharePixmapBacking;
 	screen->SetSharedPixmapBacking = info->glamor.SavedSetSharedPixmapBacking;
-#endif
 }
 
 XF86VideoAdaptorPtr amdgpu_glamor_xv_init(ScreenPtr pScreen, int num_adapt)
