@@ -54,37 +54,9 @@
 
 extern DriverRec AMDGPU;
 
-typedef enum {
-	CHIP_FAMILY_UNKNOW,
-	CHIP_FAMILY_LEGACY,
-	CHIP_FAMILY_AMDGPU,
-	CHIP_FAMILY_TAHITI,
-	CHIP_FAMILY_PITCAIRN,
-	CHIP_FAMILY_VERDE,
-	CHIP_FAMILY_OLAND,
-	CHIP_FAMILY_HAINAN,
-	CHIP_FAMILY_BONAIRE,
-	CHIP_FAMILY_KAVERI,
-	CHIP_FAMILY_KABINI,
-	CHIP_FAMILY_HAWAII,
-	CHIP_FAMILY_MULLINS,
-	CHIP_FAMILY_TOPAZ,
-	CHIP_FAMILY_TONGA,
-	CHIP_FAMILY_CARRIZO,
-	CHIP_FAMILY_FIJI,
-	CHIP_FAMILY_STONEY,
-	CHIP_FAMILY_POLARIS10,
-	CHIP_FAMILY_POLARIS11,
-	CHIP_FAMILY_LAST
-} AMDGPUChipFamily;
-
-typedef struct {
-	uint32_t pci_device_id;
-	AMDGPUChipFamily chip_family;
-} AMDGPUCardInfo;
-
 typedef struct {
 	Bool HasCRTC2;		/* All cards except original Radeon  */
+	Bool has_page_flip_target;
 
 	amdgpu_device_handle pDev;
 
@@ -93,19 +65,24 @@ typedef struct {
 	unsigned long fd_wakeup_registered;	/* server generation for which fd has been registered for wakeup handling */
 	int fd_wakeup_ref;
 	unsigned int assigned_crtcs;
+	ScrnInfoPtr primary_scrn;
+	ScrnInfoPtr secondary_scrn;
 	struct xf86_platform_device *platform_dev;
+	char *render_node;
 } AMDGPUEntRec, *AMDGPUEntPtr;
 
+extern void amdgpu_kernel_close_fd(AMDGPUEntPtr pAMDGPUEnt);
+
 extern const OptionInfoRec *AMDGPUOptionsWeak(void);
 
 extern Bool AMDGPUPreInit_KMS(ScrnInfoPtr, int);
-extern Bool AMDGPUScreenInit_KMS(SCREEN_INIT_ARGS_DECL);
-extern Bool AMDGPUSwitchMode_KMS(SWITCH_MODE_ARGS_DECL);
-extern void AMDGPUAdjustFrame_KMS(ADJUST_FRAME_ARGS_DECL);
-extern Bool AMDGPUEnterVT_KMS(VT_FUNC_ARGS_DECL);
-extern void AMDGPULeaveVT_KMS(VT_FUNC_ARGS_DECL);
-extern void AMDGPUFreeScreen_KMS(FREE_SCREEN_ARGS_DECL);
+extern Bool AMDGPUScreenInit_KMS(ScreenPtr pScreen, int argc, char **argv);
+extern Bool AMDGPUSwitchMode_KMS(ScrnInfoPtr pScrn, DisplayModePtr mode);
+extern void AMDGPUAdjustFrame_KMS(ScrnInfoPtr pScrn, int x, int y);
+extern Bool AMDGPUEnterVT_KMS(ScrnInfoPtr pScrn);
+extern void AMDGPULeaveVT_KMS(ScrnInfoPtr pScrn);
+extern void AMDGPUFreeScreen_KMS(ScrnInfoPtr pScrn);
 
-extern ModeStatus AMDGPUValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode,
+extern ModeStatus AMDGPUValidMode(ScrnInfoPtr pScrn, DisplayModePtr mode,
 				  Bool verbose, int flag);
 #endif /* _AMDGPU_PROBE_H_ */
